package org.mdp.cli;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.zip.GZIPInputStream;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.mdp.wc.CountMap;

/**
 * Tries to count all unique lines of a file in memory.
 * 
 * @author Aidan
 */
public class RunBigramCountInMemory {
	
	public static int TICKS = 1000000;
	
	public static void main(String[] args) throws IOException{
		long b4 = System.currentTimeMillis();
		
		Option inO = new Option("i", "input file");
		inO.setArgs(1);
		inO.setRequired(true);
		
		Option ingzO = new Option("igz", "input file is GZipped");
		ingzO.setArgs(0);
		
		Option helpO = new Option("h", "print help");
		
		Option kO = new Option("k", "do a top-k count (default all)");
		kO.setArgs(1);
				
		Options options = new Options();
		options.addOption(inO);
		options.addOption(ingzO);
		options.addOption(kO);
		options.addOption(helpO);

		CommandLineParser parser = new BasicParser();
		CommandLine cmd = null;

		try {
			cmd = parser.parse(options, args);
		} catch (ParseException e) {
			System.err.println("***ERROR: " + e.getClass() + ": " + e.getMessage());
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp("parameters:", options );
			return;
		}
		
		// print help options and return
		if (cmd.hasOption("h")) {
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp("parameters:", options );
			return;
		}
		
		int k = Integer.MAX_VALUE;
		if(cmd.hasOption(kO.getOpt())){
			k = Integer.parseInt(cmd.getOptionValue(kO.getOpt()));
		}
		
		String in = cmd.getOptionValue(inO.getOpt());
		InputStream is = new FileInputStream(in);
		if(cmd.hasOption(ingzO.getOpt())){
			is = new GZIPInputStream(is);
		}
		BufferedReader br = new BufferedReader(new InputStreamReader(is));

		CountMap<String> bigrams = new CountMap<String>();
		int nonUnique = 0;
		String line = null;
		while((line=br.readLine())!=null){
			nonUnique++;
			if(nonUnique % TICKS == 0){
				System.err.println("Read "+nonUnique+" non-unique bigrams");
				System.err.println("Read "+bigrams.size()+" unique bigrams");
			}
			
			bigrams.add(line);
		}
		
		System.err.println("Finished!");
		System.err.println("Read "+nonUnique+" non-unique words");
		System.err.println("Read "+bigrams.size()+" unique words");
		
		bigrams.printOrderedStats(k);
		br.close();
		
		System.err.println("Finished in "+(System.currentTimeMillis()-b4)/1000+" seconds");
	}
}

