package org.mdp.wc;

import java.io.BufferedReader;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Parses words from a file into bi-grams: two consecutive words
 * separated by a space.
 * 
 * @author Aidan
 *
 */
public class BigramParserIterator implements Iterator<String> {
	final WordParserIterator wpi;
	String next = null;
	String prevWord = null;
	
	
	public BigramParserIterator(BufferedReader br){
		wpi = new WordParserIterator(br);
		loadNext();
	}
	
	private void loadNext(){
		next = null;
		
		if(prevWord==null && wpi.hasNext()){
			prevWord = wpi.next();
		}
		
		if(prevWord!=null && wpi.hasNext()){
			String newWord = wpi.next();
			next = prevWord+" "+newWord;
			prevWord = newWord;
		}
	}

	@Override
	public boolean hasNext() {
		return next!=null;
	}

	@Override
	public String next() {
		if(!hasNext())
			throw new NoSuchElementException();
		String copyNext = this.next;
		loadNext();
		return copyNext;
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
}
